from datetime import datetime, timedelta
import logging

import requests
import urllib3

from typing import Dict, List

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

default_logger: logging.Logger = logging.getLogger(__name__)


class Job:
    def __init__(self, raw_json):
        self.type = raw_json.get("type", None)
        self.id = raw_json.get("id", None)
        self.job_type = raw_json["attributes"].get("jobType", None)
        self.jon_sub_type = raw_json["attributes"].get("jobSubType", None)
        self.policy_type = raw_json["attributes"].get("policyType", None)
        self.policy_name = raw_json["attributes"].get("policyName", None)
        self.schedule_type = raw_json["attributes"].get("scheduleType", None)
        self.schedule_name = raw_json["attributes"].get("scheduleName", None)
        self.client_name = raw_json["attributes"].get("clientName", None)
        self.control_host = raw_json["attributes"].get("controlHost", None)
        self.job_owner = raw_json["attributes"].get("jobOwner", None)
        self.job_group = raw_json["attributes"].get("jobGroup", None)
        self.backup_id = raw_json["attributes"].get("backupId", None)
        self.state = raw_json["attributes"].get("state", None)
        self.status = raw_json["attributes"].get("status", None)
        self.start_time = raw_json["attributes"].get("startTime", None)  # "2019-11-03T19:03:41.000Z"
        self.end_time = raw_json["attributes"].get("endTime", None)

    def __str__(self):
        return f"Job ({self.type}, {self.id}) - {self.state}"

    def __repr__(self):
        return self.__str__()


class Policy:
    def __init__(self, raw_json):
        self.type = raw_json.get("type", None)
        self.id = raw_json.get("id", None)

    def __str__(self):
        return f"Policy ({self.type}, {self.id})"

    def __repr__(self):
        return self.__str__()


class Alert:
    def __init__(self, raw_json):
        self.type = raw_json.get("type", None)
        self.id = raw_json.get("id", None)
        self.title = raw_json["attributes"].get("title", None)
        self.description = raw_json["attributes"].get("description", None)
        self.category = raw_json["attributes"].get("category", None)
        self.sub_category = raw_json["attributes"].get("subCategory", None)
        self.severity = raw_json["attributes"].get("severity", None)
        self.created_time = raw_json["attributes"].get("createdDateTime", None)
        self.params = raw_json["attributes"].get("params", None)
        self.policy_name: str = self.params.get("policyName", "")

    def __str__(self):
        return f"Alert ({self.type}, {self.id}) - {self.title}"

    def __repr__(self):
        return self.__str__()


class NetbackupAPI:
    def __init__(self, url: str, username: str, password: str, logger=default_logger):
        self.url = url
        self.username = username
        self.password = password
        self.logger = logger
        self.token = None

    def _make_request(self, path, method="GET", params=None, headers=None, json=None):
        url = f"{self.url}{path}"

        if not url.endswith("/login") and self.token is None:
            self.login()

        if headers is None:
            headers = {}
        headers.update({"Authorization": self.token})

        self.logger.debug(f"Making request to {url}")
        r = requests.request(method, url, params=params, headers=headers, json=json, verify=False)
        self.logger.debug(f"Got response: {r}")
        if r.status_code >= 400:
            self.logger.error(f'Got response "{r}" for url "{url}": {r.text}')
            raise requests.HTTPError(f"URL: {url}, Response: {r}")
        return r.json()

    def login(self):
        path = "/login"
        params = {"userName": self.username, "password": self.password}
        headers = {"Content-Type": "application/json"}
        r = self._make_request(path, method="POST", json=params, headers=headers)
        self.token = r["token"]

    def get_jobs(self, api_filter=None, page_limit=None, page_offset=0, sort=None) -> List[Job]:
        jobs: List[Job] = []
        path = "/admin/jobs"
        params = {
            "filter": api_filter,
            "page[limit]": page_limit,
            "page[offset]": page_offset,
            "sort": sort,
        }
        jobs_request = self._make_request(path, params=params)
        jobs.extend([Job(raw_json) for raw_json in jobs_request["data"]])
        if jobs_request["meta"]["pagination"]["pages"] > 1:
            for i in range(1, jobs_request["meta"]["pagination"]["pages"]):
                params["page[offset]"] += 100
                jobs_request = self._make_request(path, params=params)
                jobs.extend([Job(raw_json) for raw_json in jobs_request["data"]])

        return jobs

    def get_policies(self) -> List[Policy]:
        path = "/config/policies/"
        return [Policy(raw_json) for raw_json in self._make_request(path)["data"]]

    def get_alerts(self, api_filter=None, page_limit=None, page_offset=0, sort=None):
        path = "/manage/alerts"
        params = {
            "filter": api_filter,
            "page[limit]": page_limit,
            "page[offset]": page_offset,
            "sort": sort,
        }
        return [Alert(raw_json) for raw_json in self._make_request(path, params=params)["data"]]

    def app_details(self):
        path = "/appdetails"
        return self._make_request(path)

    def storage_backup_size(self):
        path = "/storage/backup-sizes"
        return self._make_request(path)

    def get_hosts(self):
        path = "/config/hosts"
        return self._make_request(path)


def main():
    import json

    logger = logging.getLogger(__name__)
    st = logging.StreamHandler()
    st.setLevel(logging.DEBUG)
    logger.addHandler(st)
    logger.setLevel(logging.DEBUG)

    with open("properties.json", "r") as f:
        config = json.load(f)

    n = NetbackupAPI(config["netbackup_url"], config["netbackup_username"], config["netbackup_password"], logger=logger,)

    # policies = n.get_policies()

    today = datetime.utcnow().replace(hour=0, second=0, minute=0, microsecond=0).strftime("%Y-%m-%dT%H:%M:%S.000Z")
    # jobs_filter = f"endTime gt {today} or state eq 'QUEUED' or state eq 'ACTIVE' or state eq 'REQUEUED'"
    # jobs = n.get_jobs(page_limit=100, api_filter=jobs_filter)

    # alert_filter = f"createdDateTime gt {today}"
    # alerts = n.get_alerts(page_limit=100, api_filter=alert_filter)

    print(n.get_hosts())


if __name__ == "__main__":
    main()
    # '2019-11-03T20:00:02.000Z'
    # '2019-11-03T17:00:00.000Z'
