from collections import Counter
from datetime import datetime
import logging

from ruxit.api.base_plugin import RemoteBasePlugin

from netbackup_api import NetbackupAPI


logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


class NetbackupPlugin(RemoteBasePlugin):

    runs = 0

    def query(self, **kwargs) -> None:

        config = kwargs["config"]
        group = self.topology_builder.create_group(f'{config["group"]}', f'{config["group"]}')
        device = group.create_device(f'{config["device"]}')

        includes = config.get("include_prefix", "").split(",")
        excludes = config.get("exclude_prefix", "").split(",")
        ignore_others = bool(config.get("ignore_others", False))

        frequency = config.get("frequency", "")
        run = True
        if frequency:
            if self.runs % int(frequency) != 0:
                run = False
        self.runs += 1

        if run:
            try:
                n = NetbackupAPI(
                    config["netbackup_url"], config["netbackup_username"], config["netbackup_password"], logger=logger
                )
                last_minute = datetime.utcnow().replace(second=0, microsecond=0).strftime("%Y-%m-%dT%H:%M:%S.000Z")

                # Get jobs details
                jobs_filter = f"endTime gt {last_minute} or state eq 'QUEUED' or state eq 'ACTIVE' or state eq 'REQUEUED'"
                jobs = n.get_jobs(page_limit=100, api_filter=jobs_filter)
                job_states = Counter(job.state for job in jobs)
                for state, value in job_states.items():
                    device.absolute("job-state", value, dimensions={"State": state})

                # Report hosts statistics
                hosts = n.get_hosts()
                device.report_property("Hosts", f'{len(hosts["hosts"])}')

                # Report policies statistics
                policies = n.get_policies()
                device.report_property("Policies", f"{len(policies)}")

                # Report alerts
                alert_filter = f"createdDateTime gt {last_minute}"
                for alert in n.get_alerts(page_limit=100, api_filter=alert_filter):
                    send_alert = True

                    if ignore_others:
                        send_alert = False

                    for include in includes:
                        if include and alert.policy_name.startswith(include):
                            send_alert = True

                    for exclude in excludes:
                        if exclude and alert.policy_name.startswith(exclude):
                            send_alert = False

                    if send_alert:
                        properties = alert.params
                        properties["Created"] = alert.created_time
                        logger.info(f"Sending event for policy {alert.policy_name}: {alert.title} - {alert.description}")
                        device.report_error_event(
                            description=f"Netbackup - {alert.description}", title=alert.title[:100], properties=properties,
                        )
                    else:
                        logger.info(
                            f"Will not send alert from policy {alert.policy_name}. Include list: {includes}. Exclude list: {excludes}. Ignore thers: {ignore_others}"
                        )
            except Exception as e:
                error = f"Netbackup Plugin - Error {e}"
                logger.exception(error)
                device.report_error_event(error, "Netbackup Plugin - Error executing")
